import 'dart:math';

import 'package:calendar_view/utils/calendar_util.dart';
import 'package:calendar_view/utils/event_type.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class CalendarController extends GetxController {
  CalendarController();

  /// 虚拟的日程数据
  final List<DateTime> mockSchedule = [
    DateTime(2022, 12, 1),
    DateTime(2022, 12, 4),
    DateTime(2022, 12, 5),
    DateTime(2022, 12, 6),
    DateTime(2022, 12, 7),
    DateTime(2022, 12, 12),
    DateTime(2022, 12, 13),
    DateTime(2022, 12, 14),
    DateTime(2022, 12, 15),
    DateTime(2022, 12, 16),
    DateTime(2022, 12, 17),
    DateTime(2022, 12, 18),
    DateTime(2022, 12, 19),
    DateTime(2022, 12, 20),
    DateTime(2022, 12, 27),
    DateTime(2022, 12, 28),
    DateTime(2022, 12, 29),
    DateTime(2022, 12, 30),
    DateTime(2022, 12, 31),
  ];

  /// 数据项：当前年份，当前月份，当前日期
  int currentYear = 2022; // 当前年份
  int currentMonth = 12; // 当前月份
  int selectedDayIndex = 0; // 当前选择的日期的下标，从0开始，范围是0-34 有时是 0-41
  int maxDaysLength = 35; // 最大的日期数组长度，有时是42
  List<Schedule> scheduleList = []; // 当前维护的日程列表【数据源】
  List<ScheduleType> scheduleTypeList = []; // 当前维护的一个日程类型列表
  List<String> needDisplayTypeNameList = []; // 维护一个需要展示的日程类型列表
  List<MiniViewDayStructure> miniViewDaysList = []; // 当前维护的迷你日历列表
  /// 日程列表与日期列表拼装，即可变成月视图大列表，将日程塞入每一天
  List<MonthViewDayStructure> get monthViewDaysList {
    final List<MonthViewDayStructure> monthViewDaysList = [];
    for (int i = 0; i < maxDaysLength; i++) {
      final MiniViewDayStructure miniViewDayStructure = miniViewDaysList[i];
      final MonthViewDayStructure monthViewDayStructure = MonthViewDayStructure(
        index: miniViewDayStructure.index,
        date: miniViewDayStructure.date,
        isToday: miniViewDayStructure.isToday,
        isCurrentMonth: miniViewDayStructure.isCurrentMonth,
        isSelected: miniViewDayStructure.isSelected,
        scheduleList: [],
      );
      monthViewDaysList.add(monthViewDayStructure);
    }
    for (final Schedule schedule in scheduleList) {
      final DateTime scheduleDate = schedule.day;
      // final int scheduleDay = scheduleDate.day;
      for (int i = 0; i < maxDaysLength; i++) {
        final MiniViewDayStructure miniViewDayStructure = miniViewDaysList[i];
        final MonthViewDayStructure monthViewDayStructure =
            monthViewDaysList[i];
        if (miniViewDayStructure.isCurrentMonth &&
            miniViewDayStructure.date == scheduleDate) {
          monthViewDayStructure.scheduleList.add(schedule);
        }
      }
    }
    return monthViewDaysList;
  }

  /// TODO:[Gavin] i18n
  String get currentYearMonth => '$currentYear年$currentMonth月';
  bool get isToday => miniViewDaysList[selectedDayIndex].isToday;
  int get calendarLines => maxDaysLength ~/ 7;

  /// 数据变更事件通知
  FEventHandler<void> onDaysListChange = FEventHandler<void>();
  @override
  void onInit() {
    super.onInit();

    /// 获取当前时间判断年月
    final DateTime now = DateTime.now();
    currentYear = now.year;
    currentMonth = now.month;
    miniViewDaysList = _countDaysList(currentYear, currentMonth);
    selectToday();
    _initMockScheduleType();
    _initMockSchedule(mockSchedule);
    // 设置mini日历的日程下标
    updateScheduleTypeList();
    _setMiniCalendarSchedule();
    // print("CalendarController 全局临时控制器初始化");
  }

  /// TODO:[Gavin] 改为真实数据接口
  /// 创建日程类型，填充到scheduleTypeList
  void _initMockScheduleType() {
    scheduleTypeList = [
      ScheduleType(
        typeName: '我的日程',
        color: Colors.red,
        isSelected: true,
      ),
      ScheduleType(
        typeName: '工作日程',
        color: Colors.blue,
        isSelected: true,
      ),
      ScheduleType(
        typeName: '会议日程',
        color: Colors.green,
        isSelected: true,
      ),
      ScheduleType(
        typeName: '其他日程',
        color: Colors.orange,
        isSelected: true,
      ),
    ];
  }

  /// 更新日程列表显示情况【只接收状态更新，不修改 scheduleTypeList】
  void updateScheduleTypeList() {
    // 更新 needDisplayTypeNameList
    needDisplayTypeNameList = [];
    for (var element in scheduleTypeList) {
      if (element.isSelected) {
        needDisplayTypeNameList.add(element.typeName);
      }
    }
    _setMiniCalendarSchedule();
    // 通知更新
    onDaysListChange.emit(this, null);
  }

  /// TODO:[Gavin] 改为真实数据接口
  void _initMockSchedule(List<DateTime> mockSchedule) {
    // 遍历mockSchedule的每一天，向 scheduleList 中添加日程，每天添加3个日程
    for (final DateTime date in mockSchedule) {
      int num = _getRandomInt(0, 20);
      for (int i = 0; i < num; i++) {
        int typeIndex = _getRandomInt(0, scheduleTypeList.length);
        final Schedule schedule = Schedule(
          day: date,
          type: scheduleTypeList[typeIndex],
          title: '${scheduleTypeList[typeIndex].typeName}-Mock标题',
          content: '日程内容',
        );
        scheduleList.add(schedule);
      }
    }
  }

  /// 取随机整数
  int _getRandomInt(int min, int max) {
    final Random random = Random();
    return min + random.nextInt(max - min);
  }

  /// 传入起始年月，返回35个日期结构体
  List<MiniViewDayStructure> _countDaysList(int year, int month) {
    maxDaysLength = _isNeedExtraRow(year, month) ? 42 : 35;
    final miniViewDaysList = <MiniViewDayStructure>[];
    final firstDay = DateTime(year, month, 1);
    final firstDayWeek = firstDay.weekday;
    final lastDay = DateTime(year, month + 1, 0);
    final lastDayOfMonth = lastDay.day;
    final lastDayOfLastMonth = DateTime(year, month, 0).day;
    final today = DateTime.now();

    /// 转换后的 firstDayWeek
    final firstDayWeekIndex = (firstDayWeek) % 7;
    int nextMonthDay = 1;

    // 上个月的日期
    for (var i = 0; i < firstDayWeekIndex; i++) {
      miniViewDaysList.add(
        MiniViewDayStructure(
          index: i,
          date: DateTime(
              year, month - 1, lastDayOfLastMonth - firstDayWeekIndex + i + 1),
          isCurrentMonth: false,
        ),
      );
    }

    // 当月的日期
    for (var i = 0;
        (i < lastDayOfMonth) && miniViewDaysList.length < maxDaysLength;
        i++) {
      miniViewDaysList.add(
        MiniViewDayStructure(
          index: i + firstDayWeekIndex,
          date: DateTime(year, month, i + 1),
          isToday:
              today.year == year && today.month == month && today.day == i + 1,
        ),
      );
    }

    while (miniViewDaysList.length < maxDaysLength) {
      miniViewDaysList.add(MiniViewDayStructure(
        index: miniViewDaysList.length,
        date: DateTime(month == 12 ? year + 1 : year,
            month == 12 ? 1 : month + 1, nextMonthDay),
        isCurrentMonth: false,
      ));
      nextMonthDay++;
    }
    return miniViewDaysList;
  }

  /// 计算是否需要显示六行日期，一般为5行，但是有时候会有6行
  bool _isNeedExtraRow(int year, int month) {
    final firstDay = DateTime(year, month, 1);
    final firstDayWeek = firstDay.weekday;
    final lastDay = DateTime(year, month + 1, 0);
    final lastDayOfMonth = lastDay.day;

    /// 转换后的firstDayWeek，%7是因为DateTime的weekday是从周日开始的，而我们的日历是从周一开始的
    final firstDayWeekIndex = (firstDayWeek) % 7;
    return firstDayWeekIndex + lastDayOfMonth > 35;
  }

  /// 通过下标选中日期，下标从0开始，允许的范围是0-34，有时是 0-42
  void handleSelectedDayByIndex(int index) {
    assert(index >= 0 && index < maxDaysLength,
        'index must be in 0-$maxDaysLength');
    assert(miniViewDaysList.length == maxDaysLength,
        'miniViewDaysList length must be maxDaysLength');
    for (var element in miniViewDaysList) {
      if (element.index == index) {
        element.isSelected = true;
      } else {
        element.isSelected = false;
      }
    }
    selectedDayIndex = index;
    onDaysListChange.emit(this, null);
  }

  /// 选中当月第一天
  void _selectFirstDay() {
    for (var element in miniViewDaysList) {
      if (element.isCurrentMonth) {
        handleSelectedDayByIndex(element.index);
        break;
      }
    }
  }

  /// 根据日期选中指定的一天
  void _selectDay(int day, List<MiniViewDayStructure> newList) {
    for (var element in newList) {
      if (element.date.day == day && element.isCurrentMonth) {
        handleSelectedDayByIndex(element.index);
        return;
      }
    }
    int nextSearchDay = day - 1;
    while (nextSearchDay > 0) {
      for (var element in newList) {
        if (element.date.day == nextSearchDay && element.isCurrentMonth) {
          handleSelectedDayByIndex(element.index);
          return;
        }
      }
      nextSearchDay--;
    }
  }

  /// 选中今天
  void selectToday() {
    /// 如果当前不是当月，则切换到当月
    final DateTime now = DateTime.now();
    if (currentYear != now.year || currentMonth != now.month) {
      currentYear = now.year;
      currentMonth = now.month;
      miniViewDaysList = _countDaysList(currentYear, currentMonth);
      _setMiniCalendarSchedule();
    }
    for (var element in miniViewDaysList) {
      if (element.isToday) {
        handleSelectedDayByIndex(element.index);
        break;
      }
    }
  }

  /// 给日期设置日程的值
  void _setMiniCalendarSchedule() {
    /// 如果 scheduleList 中存在 daysList 中的日期，则设置 hasSchedule 为 true
    for (var day in miniViewDaysList) {
      day.hasSchedule = false;
      for (var element in scheduleList) {
        var existSchedule = false;
        if (element.day.year == day.date.year &&
            element.day.month == day.date.month &&
            element.day.day == day.date.day) {
          if (_isDisplayScheduleType(element.type.typeName)) {
            existSchedule = true;
          }
        }
        if (existSchedule) {
          day.hasSchedule = true;
          break;
        }
      }
    }
  }

  /// 上个月 (keepSelectedDay 是否保持选中的日期)
  void handleLastMonth([bool keepSelectedDay = false]) {
    final currDay = miniViewDaysList[selectedDayIndex].date.day;
    if (currentMonth == 1) {
      currentMonth = 12;
      currentYear--;
    } else {
      currentMonth--;
    }
    miniViewDaysList = _countDaysList(currentYear, currentMonth);
    _setMiniCalendarSchedule();
    if (keepSelectedDay) {
      _selectDay(currDay, miniViewDaysList);
    } else {
      _selectFirstDay();
    }
    onDaysListChange.emit(this, null);
  }

  /// 下个月 (keepSelectedDay 是否保持选中的日期)
  void handleNextMonth([bool keepSelectedDay = false]) {
    final currDay = miniViewDaysList[selectedDayIndex].date.day;
    if (currentMonth == 12) {
      currentMonth = 1;
      currentYear++;
    } else {
      currentMonth++;
    }
    miniViewDaysList = _countDaysList(currentYear, currentMonth);
    _setMiniCalendarSchedule();
    if (keepSelectedDay) {
      _selectDay(currDay, miniViewDaysList);
    } else {
      _selectFirstDay();
    }
    onDaysListChange.emit(this, null);
  }

  /// 日程数据筛选器【仅筛选出需要显示的日程】
  List<Schedule> scheduleListFilter(List<Schedule> scheduleList) {
    final List<Schedule> result = [];
    for (Schedule element in scheduleList) {
      if (_isDisplayScheduleType(element.type.typeName)) {
        result.add(element);
      }
    }
    return result;
  }

  /// 是否为需要显示的日程
  bool _isDisplayScheduleType(String typeName) {
    if (needDisplayTypeNameList.contains(typeName)) {
      return true;
    }
    return false;
  }
}
