import 'package:calendar_view/calendar_controller/controller.dart';
import 'package:calendar_view/calendar_page/month_calendar/schedule_item.dart';
import 'package:calendar_view/utils/calendar_util.dart';
import 'package:calendar_view/utils/chinese_calendar_utils.dart';
import 'package:calendar_view/utils/event_type.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class MoreSchedulePopup extends StatefulWidget {
  const MoreSchedulePopup({
    Key? key,
    required this.scheduleData,
  }) : super(key: key);

  /// 日程数据
  final MonthViewDayStructure scheduleData;

  @override
  MoreSchedulePopupState createState() => MoreSchedulePopupState();
}

class MoreSchedulePopupState extends State<MoreSchedulePopup> {
  static const double _itemHeight = 28;
  List<Schedule> _scheduleDataList = [];
  CalendarController calendarController = Get.find<CalendarController>();

  @override
  void initState() {
    super.initState();
    _scheduleDataList =
        calendarController.scheduleListFilter(widget.scheduleData.scheduleList);
  }

  @override
  void didUpdateWidget(covariant MoreSchedulePopup oldWidget) {
    super.didUpdateWidget(oldWidget);
    _scheduleDataList =
        calendarController.scheduleListFilter(widget.scheduleData.scheduleList);
  }

  @override
  Widget build(BuildContext context) {
    return ListView(
      shrinkWrap: true,
      children: [
        Container(
          padding: const EdgeInsets.all(10),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              _buildScheduleHead(),
              _buildScheduleList(),
            ],
          ),
        ),
      ],
    );
  }

  Widget _buildScheduleHead() {
    return SizedBox(
      height: 35,
      child: Row(
        children: [
          Text(
            widget.scheduleData.date.day.toString(),
            style: const TextStyle(
              fontSize: 30,
            ),
          ),
          const SizedBox(width: 5),
          Text(
            _getLunarDay(widget.scheduleData.date),
            style: const TextStyle(
              fontSize: 11,
              color: Colors.grey,
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildScheduleList() {
    return ListView.builder(
      shrinkWrap: true,
      addAutomaticKeepAlives: false,
      addRepaintBoundaries: true,
      itemExtent: _itemHeight,
      itemCount: _scheduleDataList.length,
      itemBuilder: (context, index) {
        return _buildScheduleItem(index);
      },
    );
  }

  Widget _buildScheduleItem(int scheduleIndex) {
    Schedule schedule = _scheduleDataList[scheduleIndex];
    return SchedualItem(
      scheduleData: schedule,
      itemHeight: _itemHeight,
    );
  }

  String _getLunarDay(DateTime date) {
    CalendarInfo lunarDate = CalendarUtils.getInfo(date);
    return lunarDate.term ?? lunarDate.lunarDayName ?? '';
  }
}
