import 'package:calendar_view/calendar_page/month_calendar/schedule_item.dart';
import 'package:calendar_view/utils/calendar_util.dart';
import 'package:calendar_view/utils/event_type.dart';
import 'package:flutter/material.dart';

class ScheduleList extends StatefulWidget {
  const ScheduleList({
    Key? key,
    required this.maxLines,
    required this.scheduleDataList,
    required this.onTapShowMore,
  }) : super(key: key);

  /// 可容纳的最大日程行数
  final int maxLines;

  /// 每行日程的高度
  final double itemHeight = 20;

  /// 日程数据
  final List<Schedule> scheduleDataList;

  /// 点击显示更多
  final VoidCallback onTapShowMore;

  @override
  ScheduleListState createState() => ScheduleListState();
}

class ScheduleListState extends State<ScheduleList> {
  /// 容纳的数量
  int _itemCount = 0;
  @override
  void initState() {
    super.initState();
    _itemCount = widget.scheduleDataList.length > widget.maxLines
        ? widget.maxLines
        : widget.scheduleDataList.length;
  }

  @override
  void didUpdateWidget(ScheduleList oldWidget) {
    super.didUpdateWidget(oldWidget);
    _itemCount = widget.scheduleDataList.length > widget.maxLines
        ? widget.maxLines
        : widget.scheduleDataList.length;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 2),
      child: _buildScheduleTypeList(),
    );
  }

  Widget _buildScheduleTypeList() {
    return ListView.builder(
      addAutomaticKeepAlives: false,
      addRepaintBoundaries: true,
      itemExtent: widget.itemHeight,
      itemCount: _itemCount,
      itemBuilder: (context, index) {
        return _buildScheduleTypeItem(index);
      },
    );
  }

  Widget _buildScheduleTypeItem(int scheduleIndex) {
    if (scheduleIndex == _itemCount - 1) {
      if (_itemCount < widget.scheduleDataList.length) {
        return _buildMoreItemButton(
          widget.scheduleDataList.length - _itemCount + 1,
        );
      }
    }
    Schedule schedule = widget.scheduleDataList[scheduleIndex];
    return SchedualItem(
      scheduleData: schedule,
      itemHeight: widget.itemHeight,
    );
  }

  /// 构建【显示更多】按钮
  Widget _buildMoreItemButton(int moreNum) {
    return Material(
      color: Colors.transparent,
      child: InkWell(
        customBorder: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(5),
        ),
        onTap: () {
          widget.onTapShowMore.call();
        },
        hoverColor: const Color.fromARGB(255, 227, 228, 228),
        child: Container(
          height: widget.itemHeight,
          padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 0),
          color: Colors.transparent,
          child: Row(
            children: [
              Container(
                width: 6,
                height: 6,
                decoration: BoxDecoration(
                  color: Colors.transparent,
                  borderRadius: BorderRadius.circular(5),
                ),
              ),
              const SizedBox(width: 5),
              Expanded(
                child: Text(
                  breakWord('还有$moreNum项...'),
                  style: const TextStyle(
                    fontSize: 12,
                    color: Colors.black38,
                    overflow: TextOverflow.ellipsis,
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  static String breakWord(String text) {
    if (text.isEmpty) {
      return text;
    }
    String breakWord = ' ';
    for (var element in text.runes) {
      breakWord += String.fromCharCode(element);
      breakWord += '\u200B';
    }
    return breakWord;
  }
}
